/*
  Copyright 2021 The Hybridnet Authors.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

package main

import (
	"context"
	"flag"

	"github.com/spf13/pflag"
	"k8s.io/klog"
	controllerruntime "sigs.k8s.io/controller-runtime"

	"github.com/alibaba/hybridnet/pkg/feature"
	"github.com/alibaba/hybridnet/pkg/manager"
)

var (
	gitCommit string
)

func init() {
	klog.InitFlags(nil)
}

func main() {
	// parse flags
	pflag.CommandLine.AddGoFlagSet(flag.CommandLine)
	pflag.Parse()
	klog.Infof("Starting hybridnet manager with git commit: %v", gitCommit)
	klog.Infof("known features: %v", feature.KnownFeatures())

	var (
		stopCh <-chan struct{}
		mgr    *manager.Manager
		err    error
	)

	// init stop channel & context
	stopCh = controllerruntime.SetupSignalHandler()

	ctx, cancel := context.WithCancel(context.Background())
	defer cancel()

	go func() {
		<-stopCh
		cancel()
	}()

	// setup metrics http server
	go startMetricsServer()

	mgr, err = manager.NewManager()
	if err != nil {
		klog.Fatalf("fail to new hybridnet manager : %v", err)
	}

	if err = mgr.RunOrDie(ctx); err != nil {
		klog.Fatalf("fail to run hybridnet manager : %v", err)
	}
}
